"""auxi.modeling.process.materials.thermochemistry Package

The purpose of this package is to provide classes and functions for the the auxi.modeling.process.materials.thermochemistry namespace"""


__version__ = "0.2.0"
