"""auxi.modeling.financial.des Package

The purpose of this package is to provide common classes and functions for the the auxi.modeling.financial.des (double entry system) namespace"""


__version__ = "0.2.0"
