# -*- coding: utf-8 -*-
"""
This module provides the base class for assay conversion actions.

@author: Ex Mente Technologies (Pty) Ltd
"""
__version__ = "0.2.0"


class AssayConversionAction(object):
    def run(self, material):
        pass
